import hashlib
import json
import requests
import shutil

from judge_pics import judge_pics

from dateutil import parser
from lxml import html


token = ""
if token:
    headers = {"Authorization": "Token %s" % token}
else:
    print(
        "Warning: No CourtListener token used. You'll run out of free queries to the API quickly."
    )
    headers = {}


def granular_date(d, granularity):
    if not d:
        return ""

    d = parser.parse(d).date()

    GRANULARITY_YEAR = "%Y"
    GRANULARITY_MONTH = "%Y-%m"
    GRANULARITY_DAY = "%Y-%m-%d"
    if granularity == GRANULARITY_DAY:
        return "-" + d.strftime("%Y-%m-%d")
    elif granularity == GRANULARITY_MONTH:
        return "-" + d.strftime("%Y-%m")
    elif granularity == GRANULARITY_YEAR:
        return "-" + d.strftime("%Y")


def make_slug(name):
    """Hit our search engine and get back a good result. Look that up in the
    People endpoint to get glorius metadata.

    We start with a full name, so we plug that in.
    """
    # Drop middle initials
    name = name.lower()
    name = " ".join(
        [n.replace(",", "") for n in name.split() if not n.endswith(".")]
    )

    result_json = requests.get(
        "https://www.courtlistener.com/api/rest/v3/search/?type=p&name=%s&court=cand"
        % name,
        headers=headers,
    ).json()
    if result_json["count"] > 1:
        print(
            "Warning: Got back %s results for %s"
            % (
                result_json["count"],
                name,
            )
        )
        return None
    if result_json["count"] < 1:
        print("Warning: Got back no results for %s" % name)
        name_parts = name.split()
        if len(name_parts) == 2:
            return "%s-%s" % (name_parts[1].lower(), name_parts[0].lower())
        return None

    id = result_json["results"][0]["id"]
    result_json = requests.get(
        "https://www.courtlistener.com/api/rest/v3/people/?id=%s" % id,
        headers=headers,
    ).json()

    judge = result_json["results"][0]

    return "%s-%s%s" % (
        judge["name_last"].lower(),
        judge["name_first"].lower(),
        granular_date(judge["date_dob"], judge["date_granularity_dob"]),
    )


def get_hash_from_file(image):
    """Get the hash from the current file"""
    with open(image, "r") as f:
        return hashlib.sha256(f.read()).hexdigest()


def run_things():
    base_href = "http://www.cand.uscourts.gov"
    start_path = "/judges"
    start_url = base_href + start_path
    r = requests.get(start_url)
    html_tree = html.fromstring(r.text)
    html_tree.make_links_absolute(base_href)

    judge_nodes = html_tree.xpath('//section[@id="main-content"]//li')
    judge_info = []
    for node in judge_nodes:
        try:
            name = node.xpath("a/text()")[0]
            url = node.xpath("a/@href")[0]
        except IndexError:
            continue
        else:
            judge_info.append((name, url))

    for judge_name, judge_link in judge_info:
        judge_r = requests.get(judge_link)
        judge_html = html.fromstring(judge_r.text)
        judge_html.make_links_absolute(base_href)

        try:
            img_path = judge_html.xpath(
                '//div[@class = "judge_portrait"]//img/@src'
            )[0]
        except IndexError:
            print("Failed to find image for %s" % judge_link)
            continue

        img_r = requests.get(img_path, stream=True)
        if img_r.status_code == 200:
            slug = make_slug(judge_name)
            if not slug:
                continue

            with open(slug + ".jpeg", "wb") as f_img:
                img_r.raw.decode_content = True
                shutil.copyfileobj(img_r.raw, f_img)

            img_hash = get_hash_from_file(slug + ".jpeg")

            # Update judges.json
            judge_pics[slug] = {
                "artist": None,
                "date_created": None,
                "license": "Work of Federal Government",
                "source": judge_link,
                "hash": img_hash,
            }

    json.dump(
        judge_pics,
        open(os.path.join(judge_root, "judges.json"), "w"),
        sort_keys=True,
        indent=2,
    )


if __name__ == "__main__":
    run_things()



import json
import os
from pathlib import Path
from tempfile import NamedTemporaryFile
from typing import Literal

import climage
import requests

ROOT = os.path.dirname(os.path.abspath(__file__))

with Path(ROOT, "data", "people.json").open() as f:
    judges = json.load(f)


class ImageSizes:
    small: int = 128
    medium: int = 256
    large: int = 512
    original: None = None


SIZES = Literal[
    ImageSizes.small, ImageSizes.medium, ImageSizes.large, ImageSizes.original
]


def portrait(person: int, size: SIZES = None) -> str:
    """Get URL for portait on CL"""
    if not size:
        size = "orig"
    path = [x for x in judges if x["person"] == person][0]["path"]
    return f"https://portraits.free.law/v2/{size}/{path}.jpeg"


def show(person: int, size: ImageSizes = None) -> str:
    """"""
    url = portrait(person, size)
    r = requests.get(url, timeout=10)
    with NamedTemporaryFile(suffix=".jpeg") as tmp:
        with open(tmp.name, "wb") as f:
            f.write(r.content)
        output = climage.convert(tmp.name, width=40)
    return output



import argparse
import glob
import json
import os
from pathlib import Path

import boto3
from boto3.s3.transfer import S3Transfer
from PIL import Image
from resizeimage import resizeimage

sizes = ["128", "256", "512", "orig"]

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

if ROOT_DIR.split("/")[-1] != "judge_pics":
    raise "Please run update from the judge_pics directory"


def upload(file_path, aws_path, access_key, secret_key) -> None:
    """Uploads a file to an S3 bucket.

    :param file_path: File to upload
    :param aws_path: The path in the S3 bucket
    :param access_key: The access key for the S3
    :param secret_key: The secret key for the S3
    :return: None
    """
    # bucket = "dev-com-courtlistener-storage"
    bucket = "portraits.free.law"
    client = boto3.client(
        "s3",
        aws_access_key_id=access_key,
        aws_secret_access_key=secret_key,
    )
    transfer = S3Transfer(client)
    transfer.upload_file(
        file_path,
        bucket,
        aws_path,
        extra_args={"ContentType": "image/jpeg", "ACL": "public-read"},
    )
    print(f"http://{bucket}.s3-us-west-2.amazonaws.com/{aws_path}")


def resize_image(original: str, size: str) -> str:
    """Resize an image.

    :param original: The path to the original image
    :param size: The size of the image width to resize
    :return: The path to the new resized image
    """
    new_filepath = original.replace("orig", size)
    if not os.path.exists(os.path.dirname(new_filepath)):
        os.mkdir(os.path.dirname(new_filepath))
    with open(original, "rb") as f:
        img = Image.open(f)
        img = resizeimage.resize_height(img, int(size), validate=False)
        img.save(new_filepath, img.format)
    return new_filepath


def validate_json() -> bool:
    """Validate the json file contains our new judges

    :return: True if valid, else Raises an exception
    """
    with Path(ROOT_DIR, "data", "people.json").open() as f:
        judges = json.load(f)

    judges_in_json = [f'{v["path"]}.jpeg' for v in judges]

    portraits = [
        x.split("/")[-1] for x in glob.glob(f"{ROOT_DIR}/data/orig/*.jpeg")
    ]

    missing_judges = sorted(list(set(judges_in_json) ^ set(portraits)))
    if not missing_judges:
        return True

    raise Exception(f"Missing entry for: {' '.join(missing_judges)}")


def find_new_portraits(access_key: str, secret_key: str) -> list:
    """Compare s3 and local to find new portraits.

    :param access_key: Access key
    :param secret_key: Secret key
    :return: List of new portraits to process
    """
    session = boto3.Session(
        aws_access_key_id=access_key, aws_secret_access_key=secret_key
    )
    s3 = session.resource("s3")
    bucket = s3.Bucket("portraits.free.law")
    aws_portraits = [
        x.key.split("/")[-1] for x in bucket.objects.filter(Prefix="v2/orig/")
    ]
    local_portraits = [
        x.split("/")[-1] for x in glob.glob(f"{ROOT_DIR}/data/orig/*.jpeg")
    ]
    judges_to_upload = set(aws_portraits) ^ set(local_portraits)
    return sorted(list(judges_to_upload))


def main(access_key: str, secret_key: str) -> None:
    """Resize and upload new portraits to S3.

    :param access_key: The s3 access key
    :param secret_key: The s3 secret key
    :return: None
    """
    # Check for duplicate judge files
    validate_json()

    # Check for files we need to upload
    judges_to_upload = find_new_portraits(access_key, secret_key)

    # Generate new file sizes and upload them to the server
    for judge in list(judges_to_upload):
        for size in sizes:
            filepath = f"{ROOT_DIR}/data/orig/{judge}"
            aws_path = f"v2/{size}/{judge}"
            if size != "orig":
                filepath = resize_image(filepath, size)
                aws_path.replace("orig", size)
            upload(filepath, aws_path, access_key, secret_key)


if __name__ == "__main__":
    # This is mostly meant to be called from the github action but it could be
    # called locally with the correct credentials.
    parse = argparse.ArgumentParser(description="Create a new portrait")
    parse.add_argument(
        "--access-key", "-a", help="The access key", required=True
    )
    parse.add_argument(
        "--secret-key", "-s", help="The secret key", required=True
    )
    args = parse.parse_args()
    main(access_key=args.access_key, secret_key=args.secret_key)



import hashlib
import json
import re
import requests
import shutil
import subprocess
import os

from lxml import html

from judge_pics import judge_pics, judge_root


root_url = "http://dcchs.org/Portraits/"
line_re = re.compile('<a href="(.*)">(.*)</a')


def make_slug(name, path):
    last_name = re.search("(.*),", name).group(1).lower()
    first_name = re.search("([A-Z].*)[A-Z]", path).group(1).lower()
    return "%s-%s" % (last_name, first_name)


def get_artist_and_date_created(full_url):
    # Open firefox, prompt for answer, sanitize answer and return it.
    subprocess.Popen(["firefox", full_url], shell=False).communicate()
    artist = raw_input("Who made this: ")
    if artist == "":
        artist = None

    d = raw_input("When did they make it: ")
    if d == "":
        d = None
    return artist, d


def get_hash_from_file(image):
    """Get the hash from the current file"""
    with open(image, "r") as f:
        return hashlib.sha256(f.read()).hexdigest()


def run_things():
    with open("sources.txt", "r") as f:
        for line in f:
            # <a href="JesseAdkins.html">Adkins, Jesse C.</a><br>
            path = line_re.search(line).group(1)
            name = line_re.search(line).group(2)

            slug = make_slug(name, path)

            full_url = root_url + path

            r = requests.get(
                full_url, headers={"UserAgent": "freelawproject.org"}
            )
            tree = html.fromstring(r.text)
            try:
                img_path = tree.xpath('//div[@id="contentcolumn"]//img/@src')[
                    0
                ]
                full_img_src = root_url + img_path
            except IndexError:
                print("Failed to find image for %s" % full_url)
                continue

            r_img = requests.get(full_img_src, stream=True)
            if r_img.status_code == 200:
                with open(slug + ".jpeg", "wb") as f_img:
                    r_img.raw.decode_content = True
                    shutil.copyfileobj(r_img.raw, f_img)

            artist, date_created = get_artist_and_date_created(full_url)

            img_hash = get_hash_from_file(slug + ".jpeg")

            # Update judges.json
            judge_pics[slug] = {
                "artist": artist,
                "date_created": date_created,
                "license": "Work of Federal Government",
                "source": "Historical Society of the District of Columbia "
                "Circuit",
                "hash": img_hash,
            }

    json.dump(
        judge_pics,
        open(os.path.join(judge_root, "judges.json"), "w"),
        sort_keys=True,
        indent=2,
    )


if __name__ == "__main__":
    run_things()



import argparse
import hashlib
import json
import os
import subprocess
from pathlib import Path
from typing import Optional

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

if ROOT_DIR.split("/")[-1] != "judge_pics":
    raise "Please run update from the judge_pics directory"


def get_hash_from_file(image) -> str:
    """Get the hash from the current file"""

    command = [
        "exiftool",
        "-q",  # Quiet mode
        "-all=",  # Make all tags = ''
        "-overwrite_original",
        Path(ROOT_DIR, "data", "orig", image),
    ]
    subprocess.Popen(command, shell=False).communicate()

    with Path(ROOT_DIR, "data", "orig", image).open(mode="rb") as f:
        return hashlib.sha256(f.read()).hexdigest()


def update_judge_json(
    filename: Optional[str],
    person: Optional[int],
    date_created: Optional[str],
    artist: Optional[str],
    source: Optional[str],
    license: Optional[str],
) -> None:
    """Add new file to repository metadata

    :param filename: The filename
    :param person: The person ID from CL
    :param date_created: The date created
    :param artist: The artist
    :param source: The source
    :param license: The license
    :return:
    """
    with Path(ROOT_DIR, "data", "people.json").open(mode="r") as f:
        judges_json = json.load(f)

    judge = [x for x in judges_json if x["person"] == person]

    if judge:
        raise ValueError("Judge already exists")
    sha_hash = get_hash_from_file(filename)
    if any([x["hash"] == sha_hash for x in judges_json]):
        raise ValueError("Image already exists in the database")

    judges_json.append(
        {
            "hash": sha_hash,
            "license": license,
            "source": source,
            "artist": artist,
            "date_created": date_created,
            "path": filename,
            "person": person,
        }
    )

    with Path(ROOT_DIR, "data", "people.json").open(mode="w") as f:
        json.dump(judges_json, f, sort_keys=True, indent=2)


if __name__ == "__main__":
    """Before one can add to the repository you must first strip the exif data
    from the image and add the metadata of the file to the list of judges
    this method checks that the image is not alrady in the system and updates the
    json file people.json"""

    parse = argparse.ArgumentParser(description="Create a new portrait")
    parse.add_argument(
        "--filename", help="The name of the file", required=True
    )
    parse.add_argument(
        "--person", type=int, help="The Courtlistener Person ID", required=True
    )
    parse.add_argument(
        "--source",
        help="The source (url) of the image",
        required=False,
        default=None,
    )
    parse.add_argument(
        "--artist", help="The artist", required=False, default=None
    )
    parse.add_argument(
        "--license",
        help="The artist",
        required=False,
        default="Work of Federal Government",
    )
    parse.add_argument(
        "--date-created", help="The date created", required=False, default=None
    )

    update_judge_json(
        filename=parse.parse_args().filename,
        person=parse.parse_args().person,
        date_created=parse.parse_args().date_created,
        artist=parse.parse_args().artist,
        source=parse.parse_args().source,
        license=parse.parse_args().license,
    )



import os

from setuptools import setup, find_packages


VERSION = "1.2.3"
HERE = os.path.abspath(os.path.dirname(__file__))


reqs_path = HERE + "/requirements.txt"
with open(reqs_path) as reqs_file:
    reqs = reqs_file.read().splitlines()


setup(
    name="judge-pics",
    description="Database of Judge Pictures",
    version=VERSION,
    author="Mike Lissner",
    author_email="info@free.law",
    maintainer="Mike Lisser",
    maintainer_email="info@free.law",
    packages=find_packages(exclude=["tests", "judge_pics.data.orig"]),
    include_package_data=True,
    package_data={
        "judge_pics": [
            "data/judges.json",
            "data/people.json",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    test_suite="tests",
    install_requires=reqs,
)
